#region Copyright Syncfusion Inc. 2001 - 2006
//
//  Copyright Syncfusion Inc. 2001 - 2006. All rights reserved.
//
//  Use of this code is subject to the terms of our license.
//  A copy of the current license can be obtained at any time by e-mailing
//  licensing@syncfusion.com. Any infringement will be prosecuted under
//  applicable laws. 
//
#endregion

using System;
using System.Drawing;
using System.Collections;
using System.ComponentModel;
using System.Windows.Forms;
using System.Data;
using System.Collections.Specialized ;
using System.IO;

using Syncfusion.Windows.Forms.Tools;
using Syncfusion.Windows.Forms.Tools.XPMenus;
using Syncfusion.Runtime.Serialization;

namespace VSDemo
{
	public interface IEditorMainFrame
	{
		ListBarItem MostRecentlyUsedFilesListItem{get;}
		BarItem OpenFilesBarItem{get;}
		BarManager BarManager{get;}
		Form CreateDocumentFromFileName(string filename);
		bool SaveFile(IEditor childEditor);
	}

	public interface IEditor
	{
		bool Dirty{get;}
		string FileName{get; set;}
		void Save();
	}

	public class MDIChildEditorsManager
	{
		protected Form mainFrame;
		protected int mruCount;
		private static string MRUFilesInfoTag = "MRUInfo";
		private Hashtable formsFromFilename = new Hashtable();

		private System.Windows.Forms.OpenFileDialog openFileDialog1;
		private System.Windows.Forms.SaveFileDialog saveFileDialog1;
		private Hashtable fileLessForms = new Hashtable();

		public IEditorMainFrame EditorMainFrame
		{
			get{return this.mainFrame as IEditorMainFrame;}
		}

		public MDIChildEditorsManager(Form mainFrame, int mruCount, string openFileDialogFilter, string openFileDialogTitle)
		{
			if(!(mainFrame is IEditorMainFrame))
				throw new ArgumentException("A Form type expected", "mainFrame");

			this.mainFrame = mainFrame;
			this.mruCount = mruCount;

			this.openFileDialog1 = new System.Windows.Forms.OpenFileDialog();
			this.openFileDialog1.Filter = openFileDialogFilter;
			this.openFileDialog1.FilterIndex = 0;
			this.openFileDialog1.Title = openFileDialogTitle;

			this.saveFileDialog1 = new System.Windows.Forms.SaveFileDialog();
			this.saveFileDialog1.Filter = openFileDialogFilter;

			this.Init();
			this.InitMRUInfoFromStorage();
		}

		private void openBarItem_Click(object sender, System.EventArgs e)
		{
			this.OpenFile();
		}

		public bool IsNewFile(Form childForm)
		{
			if(this.fileLessForms[childForm] != null)
				return true;
			else
				return false;
		}

		public string GetNewFileNameForForm(Form childForm, int filterIndex)
		{
			this.saveFileDialog1.FilterIndex = filterIndex;
			string fileName = String.Empty;
			this.saveFileDialog1.FileName = fileName;
			if(this.saveFileDialog1.ShowDialog() == DialogResult.OK)
			{
				fileName = this.saveFileDialog1.FileName;
				if(this.formsFromFilename[fileName] != null)
				{
					MessageBox.Show("File is currently being used, cannot be overwritten");
					fileName = String.Empty;
				}
				else
				{
					this.fileLessForms.Remove(childForm);
					this.formsFromFilename[fileName] = childForm;
					this.AddMRUEntry(fileName, childForm);
				}
			}
			return fileName;
		}

		public void OpenFile()
		{
			string filename = String.Empty;
			if(this.openFileDialog1.ShowDialog() == DialogResult.OK)
			{
				filename = this.openFileDialog1.FileName;
				filename = filename.ToLower();
			}
			if(filename != String.Empty)
				this.AddDocumentFromFileName(filename);	
		}

		private void MRUListItem_Click(object sender, System.EventArgs e)
		{
			ListBarItemClickedEventArgs args = e as ListBarItemClickedEventArgs;

			ListBarItem listBarItem = this.EditorMainFrame.MostRecentlyUsedFilesListItem;
			Form childForm = null;
			if(listBarItem.Tags.Count > args.IndexClicked &&
				listBarItem.Tags[args.IndexClicked] != null)
			{
				childForm = listBarItem.Tags[args.IndexClicked] as Form;
				childForm.Activate();

				// Move the entry to front.
				string caption = listBarItem.ChildCaptions[args.IndexClicked];
				listBarItem.ChildCaptions.RemoveAt(args.IndexClicked);
				listBarItem.ChildCaptions.Insert(0, caption);
				listBarItem.Tags.RemoveAt(args.IndexClicked);
				listBarItem.Tags.Insert(0, childForm);
			}
			else
			{
				string filename = listBarItem.ChildCaptions[args.IndexClicked];
				listBarItem.ChildCaptions.RemoveAt(args.IndexClicked);
				if(listBarItem.Tags.Count > args.IndexClicked)
					listBarItem.Tags.RemoveAt(args.IndexClicked);
				this.AddDocumentFromFileName(filename);
			}
		}

		public virtual void AddDocumentFromFileName(string filename)
		{
			if(!File.Exists(filename))
			{
				MessageBox.Show("File: " + filename + " does not exist.");
				return;
			}

			if(this.formsFromFilename[filename] != null)
			{
				Form form = this.formsFromFilename[filename] as Form;
				form.Activate();
				return;
			}

			Form childForm = this.EditorMainFrame.CreateDocumentFromFileName(filename);
			if(childForm != null)
			{
				this.formsFromFilename[filename] = childForm;
				this.AddDocument(childForm);
				this.AddMRUEntry(filename, childForm);
			}
		}
		private void AddDocument(Form newForm) 
		{
			newForm.MdiParent = this.mainFrame;
			newForm.Show();
			newForm.Closing += new CancelEventHandler(this.ChildClosing);
		}
		public void AddNewDocument(Form newForm)
		{
			this.fileLessForms[newForm] = 1;
			this.AddDocument(newForm);
		}
		private void AddMRUEntry(string filename, Form form)
		{
			if(this.EditorMainFrame == null 
				|| this.EditorMainFrame.MostRecentlyUsedFilesListItem == null)
				return;

			ListBarItem listBarItem = this.EditorMainFrame.MostRecentlyUsedFilesListItem;

			int foundIndex = listBarItem.ChildCaptions.IndexOf(filename);
			if(foundIndex != -1)
			{
				listBarItem.ChildCaptions.RemoveAt(foundIndex);

				if(listBarItem.Tags.Count > foundIndex)
					listBarItem.Tags.RemoveAt(foundIndex);
			}

			if(listBarItem.ChildCaptions.Count >= this.mruCount)
				listBarItem.ChildCaptions.RemoveAt(this.mruCount - 1);
		
			if(listBarItem.Tags.Count >= this.mruCount)
				listBarItem.Tags.RemoveAt(this.mruCount - 1);

			listBarItem.ChildCaptions.Insert(0, filename);
			listBarItem.Tags.Insert(0, form);

			form.Disposed += new EventHandler(this.ChildFormDisposed);
		}

		private void ChildClosing(object sender, CancelEventArgs e)
		{
			IEditor editorForm = sender as IEditor;
			if(editorForm != null)
			{
				bool close = false;
				if(editorForm.Dirty)
				{
					DialogResult dlgResult = MessageBox.Show("Save Changes?", "", MessageBoxButtons.YesNoCancel);
					if(dlgResult == DialogResult.Yes)
					{
						if(this.EditorMainFrame.SaveFile(editorForm))
							close = true;
					}
					else if(dlgResult == DialogResult.No)
						close = true;
				}
				else close = true;

				if(!close)
					e.Cancel = true;
			}
		}

		private void ChildFormDisposed(object sender, EventArgs e)
		{
			Form form = sender as Form;

			form.Closing -= new CancelEventHandler(this.ChildClosing);

			ListBarItem listBarItem = this.EditorMainFrame.MostRecentlyUsedFilesListItem;
			for(int i = 0; i < listBarItem.Tags.Count; i++)
			{
				object tag = listBarItem.Tags[i];
				if(tag == form)
					listBarItem.Tags[i] = null;
			}

			// Remove mapping.
			foreach(string filename in this.formsFromFilename.Keys)
			{
				if(form == this.formsFromFilename[filename])
				{
					this.formsFromFilename.Remove(filename);
					break;
				}
			}
		}

		protected virtual void Init()
		{
			if(this.EditorMainFrame != null)
			{
				if(this.EditorMainFrame.MostRecentlyUsedFilesListItem != null)
					this.EditorMainFrame.MostRecentlyUsedFilesListItem.Click += new EventHandler(this.MRUListItem_Click);

				if(this.EditorMainFrame.OpenFilesBarItem != null)
					this.EditorMainFrame.OpenFilesBarItem.Click += new System.EventHandler(this.openBarItem_Click);
			}

			this.mainFrame.Closing += new System.ComponentModel.CancelEventHandler(this.EditorMainFrame_Closing);			
		}

		protected virtual void EditorMainFrame_Closing(object sender, System.ComponentModel.CancelEventArgs e)
		{
			this.SaveMRUInfoIntoStorage();
		}

		private void SaveMRUInfoIntoStorage()
		{
			if(this.EditorMainFrame != null)
			{
				AppStateSerializer serializer = AppStateSerializer.GetSingleton();
				serializer.SerializeObject(MRUFilesInfoTag, 
					this.EditorMainFrame.MostRecentlyUsedFilesListItem.ChildCaptions, true);
			}
		}

		private void InitMRUInfoFromStorage()
		{
			if(this.EditorMainFrame != null)
			{
				AppStateSerializer serializer = AppStateSerializer.GetSingleton();
				object objectOut = serializer.DeserializeObject(MRUFilesInfoTag);
				if(objectOut != null)
				{
					StringCollection collection = objectOut as StringCollection;
					this.EditorMainFrame.MostRecentlyUsedFilesListItem.ChildCaptions.Clear();
					foreach(string item in collection)
						this.EditorMainFrame.MostRecentlyUsedFilesListItem.ChildCaptions.Add(item);
				}
			}
		}
	}
}